@@ -15,9 +15,17 @@
 import inspect
 import pathlib
 from textwrap import dedent
+import tempfile
 import typing
+import yaml
+import weakref
 
-from ops import charm, framework, model
+from ops import (
+    charm,
+    framework,
+    model,
+    storage,
+)
 
 
 # OptionalYAML is something like metadata.yaml or actions.yaml. You can
@@ -65,14 +73,17 @@ class Harness:
         self._charm_cls = charm_cls
         self._charm = None
         self._charm_dir = 'no-disk-path'  # this may be updated by _create_meta
+        self._lazy_resource_dir = None
         self._meta = self._create_meta(meta, actions)
         self._unit_name = self._meta.name + '/0'
         self._framework = None
         self._hooks_enabled = True
         self._relation_id_counter = 0
         self._backend = _TestingModelBackend(self._unit_name, self._meta)
-        self._model = model.Model(self._unit_name, self._meta, self._backend)
-        self._framework = framework.Framework(":memory:", self._charm_dir, self._meta, self._model)
+        self._model = model.Model(self._meta, self._backend)
+        self._storage = storage.SQLiteStorage(':memory:')
+        self._framework = framework.Framework(
+            self._storage, self._charm_dir, self._meta, self._model)
 
     @property
     def charm(self) -> charm.CharmBase:
@@ -93,6 +104,16 @@ class Harness:
         """Return the Framework that is being driven by this Harness."""
         return self._framework
 
+    @property
+    def _resource_dir(self) -> pathlib.Path:
+        if self._lazy_resource_dir is not None:
+            return self._lazy_resource_dir
+
+        self.__resource_dir = tempfile.TemporaryDirectory()
+        self._lazy_resource_dir = pathlib.Path(self.__resource_dir.name)
+        self._finalizer = weakref.finalize(self, self.__resource_dir.cleanup)
+        return self._lazy_resource_dir
+
     def begin(self) -> None:
         """Instantiate the Charm and start handling events.
 
@@ -118,7 +139,7 @@ class Harness:
         # Note: jam 2020-03-01 This is so that errors in testing say MyCharm has no attribute foo,
         # rather than TestCharm has no attribute foo.
         TestCharm.__name__ = self._charm_cls.__name__
-        self._charm = TestCharm(self._framework, self._framework.meta.name)
+        self._charm = TestCharm(self._framework)
 
     def _create_meta(self, charm_metadata, action_metadata):
         """Create a CharmMeta object.
@@ -149,6 +170,40 @@ class Harness:
 
         return charm.CharmMeta.from_yaml(charm_metadata, action_metadata)
 
+    def add_oci_resource(self, resource_name: str,
+                         contents: typing.Mapping[str, str] = None) -> None:
+        """Add oci resources to the backend.
+
+        This will register an oci resource and create a temporary file for processing metadata
+        about the resource. A default set of values will be used for all the file contents
+        unless a specific contents dict is provided.
+
+        Args:
+            resource_name: Name of the resource to add custom contents to.
+            contents: Optional custom dict to write for the named resource.
+        """
+        if not contents:
+            contents = {'registrypath': 'registrypath',
+                        'username': 'username',
+                        'password': 'password',
+                        }
+        if resource_name not in self._meta.resources.keys():
+            raise RuntimeError('Resource {} is not a defined resources'.format(resource_name))
+        if self._meta.resources[resource_name].type != "oci-image":
+            raise RuntimeError('Resource {} is not an OCI Image'.format(resource_name))
+        resource_dir = self._resource_dir / resource_name
+        resource_dir.mkdir(exist_ok=True)
+        resource_file = resource_dir / "contents.yaml"
+        with resource_file.open('wt', encoding='utf8') as resource_yaml:
+            yaml.dump(contents, resource_yaml)
+        self._backend._resources_map[resource_name] = resource_file
+
+    def populate_oci_resources(self) -> None:
+        """Populate all OCI resources."""
+        for name, data in self._meta.resources.items():
+            if data.type == "oci-image":
+                self.add_oci_resource(name)
+
     def disable_hooks(self) -> None:
         """Stop emitting hook events when the model changes.
 
@@ -256,6 +311,16 @@ class Harness:
         """Read the workload version that was set by the unit."""
         return self._backend._workload_version
 
+    def set_model_name(self, name: str) -> None:
+        """Set the name of the Model that this is representing.
+
+        This cannot be called once begin() has been called. But it lets you set the value that
+        will be returned by Model.name.
+        """
+        if self._charm is not None:
+            raise RuntimeError('cannot set the Model name after begin()')
+        self._backend.model_name = name
+
     def update_relation_data(
             self,
             relation_id: int,
@@ -418,6 +483,7 @@ class _TestingModelBackend:
     def __init__(self, unit_name, meta):
         self.unit_name = unit_name
         self.app_name = self.unit_name.split('/')[0]
+        self.model_name = None
         self._calls = []
         self._meta = meta
         self._is_leader = None
